#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""

Utilitaire de mise à jour des variables
pour les versions >= 2.4.1

"""
from creole.upgrade import log
from pyeole.i18n import i18n
from tiramisu.setting import owners
_ = i18n('creole')

class Upgrade():
    """
    Méthodes pour la mise à niveau des variables
    """
    def __init__(self, config):
        owner = 'upgrade'
        if owner not in dir(owners):
            owners.addowner(owner)
        self.config = config
        self.owner = getattr(owners, owner)
        self.unknown_options = config.impl_get_information(u'unknown_options')

    def get_old_value(self, old_variable):
        """
        Retourne la valeur d'une variable "disparue"
        """
        return self.unknown_options[old_variable][u'val']

    def get_value(self, variable, default=None):
        """
        Retourne la valeur d'une variable "connue"
        """
        try:
            path = self.get_path(variable)
        except AttributeError:
            return default
        return self.config.getattr(path,
                                   force_permissive=True)

    def get_path(self, variable):
        """
        Retourne le chemin complet d'une variable
        """
        return self.config.find_first(byname=variable, type_='path')

    def modify_owner(self, path):
        """
        Modifie le propriétaire d'une variable
        """
        option = self.config.unwrap_from_path(path)
        self.config.cfgimpl_get_values().setowner(option,
                                                  self.owner)

    def set_value(self, variable, value):
        """
        Modifie la valeur d'une variable
        """
        path = self.get_path(variable)
        try:
            self.config._setattr(path, value,
                                 force_permissive=True)
            self.modify_owner(path)
        except ValueError:
            option = self.config.unwrap_from_path(path)
            self.config.cfgimpl_get_settings()[option].append('load_error')

    def append_value(self, variable, value):
        """
        Ajoute une valeur à une variable multi
        """
        path = self.get_path(variable)
        multi = self.config.getattr(path,
                                    force_permissive=True)
        multi.append(value)
        self.modify_owner(path)

    def modify_last_value(self, variable, value):
        """
        Modifie la dernière valeur d'une variable multi
        """
        path = self.get_path(variable)
        multi = self.config.getattr(path,
                            force_permissive=True)
        multi[-1] = value
        self.modify_owner(path)

    def move(self, old_variable, new_variable):
        """
        Déplace les données d'une variable "disparue"
        vers une nouvelle variable
        """
        if old_variable in self.unknown_options:
            value = self.unknown_options[old_variable][u'val']
            self.set_value(new_variable, value)
            del(self.unknown_options[old_variable])
            log.info(_(u"Variable {0} has been renamed to {1}").format(old_variable, new_variable))


class Upgrade240_241(Upgrade):
    """
    Mise à jour d'une configuration
    de 2.4.0 vers 2.4.1
    """
    def run(self):
        """
        Lancement des traitements
        """
        log.info(_(u"Starting {0} to {1} upgrade").format('2.4.0', '2.4.1'))

        # renommage des variables "era_proxy_bypass"
        for i in range(1, 5):
            self.move('era_proxy_bypass_eth{0}'.format(i), 'proxy_bypass_network_eth{0}'.format(i))

        # fusion des variables "proxy_bypass" et "wpad_exclude"
        if 'adresse_ip_wpad_exclude' in self.unknown_options:
            for idx, value in enumerate(self.get_old_value('adresse_ip_wpad_exclude')):
                interface = self.get_old_value('interface_wpad_exclude')[idx]
                if interface == 'Toutes':
                    interfaces = range(1, 5)
                elif int(interface) in range(1, 5):
                    interfaces = [interface]
                else:
                    log.error(_(u"Invalid value : {0} in old variable {1}").format(interface, 'interface_wpad_exclude'))
                    continue
                for i in interfaces:
                    self.append_value('proxy_bypass_network_eth{0}'.format(i), value)
                    self.modify_last_value('proxy_bypass_netmask_eth{0}'.format(i), self.get_old_value('adresse_netmask_wpad_exclude')[idx])
            del(self.unknown_options['adresse_ip_wpad_exclude'])
            del(self.unknown_options['adresse_netmask_wpad_exclude'])
            del(self.unknown_options['interface_wpad_exclude'])

        # passage à oui des variables "proxy_bypass_ethX" si nécessaire
        for i in range(1, 5):
            if len(self.get_value('proxy_bypass_network_eth{0}'.format(i), [])) > 0:
                self.set_value('proxy_bypass_eth{0}'.format(i), u'oui')

        # transfert des variables nom_domaine_wpad_exclude
        if 'nom_domaine_wpad_exclude' in self.unknown_options:
            for idx, value in enumerate(self.get_old_value('nom_domaine_wpad_exclude')):
                interface = self.get_old_value('nom_interface_wpad_exclude')[idx]
                if interface == 'Toutes':
                    interfaces = range(1, 5)
                elif int(interface) in range(1, 5):
                    interfaces = [interface]
                else:
                    log.error(_(u"Invalid value : {0} in old variable {1}").format(interface, 'nom_interface_wpad_exclude'))
                    continue
                for i in interfaces:
                    self.append_value('proxy_bypass_domain_eth{0}'.format(i), value)
            del(self.unknown_options['nom_domaine_wpad_exclude'])
            del(self.unknown_options['nom_interface_wpad_exclude'])

def upgrade240_241(config):
    """
    Lancement de la mise à niveau
    de 2.4.0 vers 2.4.1
    """
    upgrade = Upgrade240_241(config)
    upgrade.run()
